/*
 * Copyright, 2013, Teradek
 *
 * All rights reserved.
 * Contact Teradek for licensing.
 */

#ifndef _YOUTUBELIVE_API_H_
#define _YOUTUBELIVE_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <curl/curl.h>

#define YTL_OAUTH_ENDPOINT      "https://accounts.google.com/o/oauth2"
#define YTL_API_ENDPOINT        "https://www.googleapis.com/youtube/v3"

#define YTL_API_CONF_FILE       "/etc/ytl/api"
#define YTL_API_AUTH_FILE       "/etc/oauth/ytl"
#define YTL_API_TMP_AUTH_DIR    "/tmp/oauth"
#define YTL_API_TMP_AUTH_FILE   "/tmp/oauth/ytl"
#define YTL_DEAUTHORIZE_HOOK    "/usr/share/tdapi/youtubelive_authentication.lua"

#define YTL_CLIENT_ID           "104011399421.apps.googleusercontent.com"
#define YTL_CLIENT_SECRET       "VufM3Bwto-Y7ihWGAa0biHsE"

#define YTL_CONF_MAX_LEN        256

#define YTL_API_CONNECT_TIMEOUT 30
#define YTL_API_REQUEST_TIMEOUT 20

typedef struct
{
    CURL *curl;
} YTL_API_HANDLE;

typedef struct
{
    char id[YTL_CONF_MAX_LEN];
    char eventStatus[YTL_CONF_MAX_LEN];
    char streamId[YTL_CONF_MAX_LEN];

    char title[YTL_CONF_MAX_LEN];
    char scheduledStartTime[YTL_CONF_MAX_LEN];
    int preview;
    unsigned int broadcastStreamDelayMs;
    int enableDvr;
    int enableContentEncryption;
    int enableEmbed;
    int recordFromStart;
    int startWithSlate;
} YTL_API_BROADCAST;

typedef struct
{
    char id[YTL_CONF_MAX_LEN];
    char title[YTL_CONF_MAX_LEN];
    char format[YTL_CONF_MAX_LEN];
    char name[YTL_CONF_MAX_LEN];
    char status[YTL_CONF_MAX_LEN];
    char url[2][YTL_CONF_MAX_LEN];
} YTL_API_STREAM;

enum
{
    YTL_API_ERR_NONE = 0,
    YTL_API_ERR_NOMEM = -1,
    YTL_API_ERR_UNAUTHORIZED = -2,
    YTL_API_ERR_REAUTHORIZED = -3,
    YTL_API_ERR_INVALID_PARAM = -4,
    YTL_API_ERR_API_ERROR = -5
};

YTL_API_HANDLE *ytl_api_create_handle(void);
void ytl_api_destroy_handle(YTL_API_HANDLE *handle);
void ytl_api_set_verbose(int verbose);
void ytl_api_set_timeout(int connect_timeout, int request_timeout);
void ytl_api_get_timeout(int *connect_timeout, int *request_timeout);
int ytl_api_set_endpoints(char *api_endpt, char *oauth_endpt);
int ytl_api_check_respcode(int ret, long httpcode);
void ytl_api_request_free(char **reply);

char *ytl_api_conf_get(char *key, char *value, char *conffile);
int ytl_api_conf_set(char *key, char *value, char *conffile);

int ytl_api_request_device_code(YTL_API_HANDLE *h, char *client_id, char *scope, char **reply);
int ytl_api_check_authorization(YTL_API_HANDLE *h, char *client_id, char *client_secret, char *code, char **reply);
int ytl_api_refresh_access_token(YTL_API_HANDLE *h, char *client_id, char *client_secret, char *refresh_token, char **reply);
int ytl_api_save_device_code(char *reply);
int ytl_api_remove_device_code(void);
int ytl_api_save_access_token(char *reply);
int ytl_api_deauthorize(char **reply);

int ytl_api_channels_list(YTL_API_HANDLE *h, char *access_token, char *part, char *filter_key, char *filter_val, int maxResults, char **reply);

int ytl_api_live_broadcasts_list(YTL_API_HANDLE *h, char *access_token, char *part, char *filter_key, char *filter_val, int maxResults, char *pageToken, char **reply);
int ytl_api_live_broadcasts_info(YTL_API_HANDLE *h, char *access_token, char *id, YTL_API_BROADCAST *br, char *errorStr);
int ytl_api_live_broadcasts_bind(YTL_API_HANDLE *h, char *access_token, char *part, char *id, char *streamId, char **reply);
int ytl_api_live_broadcasts_bind_error(YTL_API_HANDLE *h, char *access_token, char *brId, char *streamId, char *errorStr);
int ytl_api_live_broadcasts_transition(YTL_API_HANDLE *h, char *access_token, char *part, char *id, char *broadcastStatus, char **reply);
int ytl_api_live_broadcasts_transition_error(YTL_API_HANDLE *h, char *access_token, char *id, char *broadcastStatus, char *errorStr);
int ytl_api_live_broadcasts_update(YTL_API_HANDLE *h, char *access_token, char *part, char *jsonData, char **reply);
int ytl_api_live_broadcasts_update_error(YTL_API_HANDLE *h, char *access_token, YTL_API_BROADCAST *br, char *errorStr);

int ytl_api_live_streams_list(YTL_API_HANDLE *h, char *access_token, char *part, char *filter_key, char *filter_val, int maxResults, char *pageToken, char **reply);
int ytl_api_live_streams_info(YTL_API_HANDLE *h, char *access_token, char *id, YTL_API_STREAM *stream, char *errorStr);
int ytl_api_live_streams_find(YTL_API_HANDLE *h, char *access_token, char *title, char *format, YTL_API_STREAM *stream, char *errorStr);
int ytl_api_live_streams_insert(YTL_API_HANDLE *h, char *access_token, char *part, char *title, char *format, char *ingestionType, char **reply);
int ytl_api_live_streams_create(YTL_API_HANDLE *h, char *access_token, char *title, char *format, YTL_API_STREAM *stream, char *errorStr);
int ytl_api_live_streams_update(YTL_API_HANDLE *h, char *access_token, char *part, char *id, char *title, char *format, char *ingestionType, char **reply);
int ytl_api_live_streams_delete(YTL_API_HANDLE *h, char *access_token, char *id, char **reply);

#ifdef __cplusplus
}
#endif

#endif /* _YOUTUBELIVE_API_H_ */
